#!/usr/bin/python
import os
import sys
import re
import getopt
import shutil
import subprocess
from clcommon import clcagefs

PASSENGER_TEMP_DIR = '/var/passenger'


def get_pattern():
    return re.compile(
        r"""LoadModule\s+passenger_module.*?<IfModule mod_passenger\.c>.*?</IfModule>""",
        re.M|re.S)


def clear_folder(path):
    for item in os.listdir(path):
        full_path = os.path.join(path, item)
        try:
            if os.path.isdir(full_path):
                shutil.rmtree(full_path)
            else:
                os.unlink(full_path)
        except Exception:
            print >>sys.stderr, "Cannot remove %s. Skipping" % (full_path)
            continue


def inflate(path, ar_path='/usr/share/alt-mod-passenger/archives', ar_suffix='.tar.gz'):
    program = 'passenger'
    archive_full_path = os.path.join(ar_path, program+ar_suffix)
    clear_folder(path)
    cmd = ['/bin/tar', 'zxf', archive_full_path, '-C', path, '--strip-components', '1']
    returncode = subprocess.call(cmd)
    if returncode:
        raise SystemExit("Cannot inflate [%d] : %s" % (returncode, ' '.join(cmd)))


def add_to_apache(path, conf_path, dir_token='/buildout'):
    pos = path.find(dir_token)
    patt = get_pattern()
    if pos == -1:
        raise SystemExit("Cannot deduce module directory")
    dir_path = path[:pos]
    try:
        f = open(conf_path, 'r+')
        try:
            data = f.read()
            rv = re.match(patt, data)
            if rv is None:
                conf_tmpl = ('\nLoadModule passenger_module %s\n'
                        '<IfModule mod_passenger.c>\n'
                        '    PassengerRoot %s\n'
                        '    PassengerDefaultRuby /usr/bin/ruby\n'
                        '    PassengerTempDir %s\n'
                        '</IfModule>')
                f.write(conf_tmpl % (path, dir_path, PASSENGER_TEMP_DIR))
        finally:
            f.close()
    except (OSError, IOError):
        raise SystemExit("Cannot add passenger to apache config")
    
    
def delete_from_apache(path):
    patt = get_pattern()
    try:
        f = open(path, 'r+')
        try:
            data = f.read()
            data = re.sub(patt, '', data)
            f.seek(0)
            f.truncate()
            f.write(data)
        finally:
            f.close()
    except (OSError, IOError):
        raise SystemExit("Cannot read config")


def restart_apache():
    cmd = ['/sbin/service', 'httpd', 'restart']
    returncode = subprocess.call(cmd,
                                 stdout=open('/dev/null','a'),
                                 stderr=open('/dev/null','a'))
    if returncode:
        raise SystemExit("Cannot restart apache [%d] : %s" % (returncode, ' '.join(cmd)))


def setup_passenger_temp_dir():
    try:

        if clcagefs.is_cagefs_present():
            clcagefs.setup_mount_dir_cagefs(PASSENGER_TEMP_DIR,
                                            added_by="alt-mod-passenger",
                                            mode=01777)
        else:
            if not os.path.isdir(PASSENGER_TEMP_DIR):
                os.mkdir(PASSENGER_TEMP_DIR)
            os.chmod(PASSENGER_TEMP_DIR, 01777)

        open("/etc/profile.d/alt_mod_passenger.sh", "w").write("export PASSENGER_TMPDIR=%s\n"
                                                               % PASSENGER_TEMP_DIR)
    except clcagefs.CagefsMpConflict, e:
        raise SystemExit("Could not setup PassengerTempDir [%s]" % str(e))
    except EnvironmentError, e:
        raise SystemExit("Could not setup PassengerTempDir [%s]" % str(e))


def add_passenger_install_dir(path):

    if not clcagefs.is_cagefs_present():
        return
    try:
        clcagefs.setup_mount_dir_cagefs(path,
                                        added_by="alt-mod-passenger",
                                        prefix='!')

    except clcagefs.CagefsMpConflict, e:
        raise SystemExit("Could not setup PassengerTempDir [%s]" % str(e))
    except EnvironmentError, e:
        raise SystemExit("Could not setup PassengerTempDir [%s]" % str(e))


def setup_passenger_cagefs(path, dummy=None):
    setup_passenger_temp_dir()
    add_passenger_install_dir(path)


def install(path, conf_path):
    # we try to create the dir beforehand to avoid apache module compilation error
    apache_configs_path = '/usr/local/apache/conf/conf.d'
    try:
        os.mkdir(apache_configs_path)
    except OSError:
        pass
    interpreter_path = '/opt/alt/ruby21/bin/ruby'
    exec_path = 'bin/passenger-install-apache2-module'
    mod_path = 'buildout/apache2/mod_passenger.so'
    inflate(path)
    os.chdir(path)
    if os.path.exists(interpreter_path) and os.path.isfile(exec_path):
        cmd = [interpreter_path, exec_path, '--auto', '--languages', 'python,ruby']
        returncode = subprocess.call(cmd,
                                     stdout=open('/dev/null','a'),
                                     stderr=open('/dev/null', 'a'))
        if returncode:
            raise SystemExit("Could not compile module [%d] : %s" % (returncode, ' '.join(cmd)))
    if not os.path.exists(mod_path):
        raise SystemExit("Could not find module")
    mod_full_path = os.path.join(path, mod_path)
    add_to_apache(mod_full_path, conf_path)
    setup_passenger_cagefs(path)
    #restart_apache()


def uninstall(path, conf_path):
    clear_folder(path)
    delete_from_apache(conf_path)


def usage(home, conf):
    print 'Usage: %s OPTIONS' % (sys.argv[0],)
    print ''
    print '-i| --install : installs the module'
    print '-d| --delete  : removes the module'
    print '-h| --help    : shows this help'
    print '-p| --path    : path to look for sources. Default path is %s' % (home,)
    print '-c| --config  : path to apache conf file. Default is %s' % (conf,)
    print '--setup-passenger-cagefs : configure passenger with CageFS'

def main():
    path = '/opt/passenger'
    apache_conf_path = '/usr/local/apache/conf/includes/pre_main_global.conf'
    action = 'install'
    dispatch = {
        'install': install,
        'delete':  uninstall,
        'setup_passenger_cagefs':  setup_passenger_cagefs}
    opts, args = getopt.getopt(sys.argv[1:],
        'idhp:c:',
        ['install', 'delete', 'path=', 'config=', 'help', "setup-passenger-cagefs"])

    for o, a in opts:
        if o in ['-h', '--help']:
            usage(path, apache_conf_path)
            sys.exit(0)
        elif o in ['-i', '--install']:
            action = 'install'
        elif o in ['-d', '--delete']:
            action = 'delete'
        elif o in ['-p', '--path']:
            path = a
        elif o in ['-c', '--config']:
            apache_conf_path = a
        elif o == '--setup-passenger-cagefs':
            action = 'setup_passenger_cagefs'

    dispatch[action](path, apache_conf_path)

if __name__ == '__main__':
    main()
